@@ -16,7 +16,27 @@ Log::Handler - Log messages to several outputs.
         newline  => 1,
     });
 
-    $log->warning("a warinng here");
+    $log->warning("warn message");
+
+Or
+
+    # init myapp logger with accessor LOG
+    package MyApp;
+    use Log::Handler myapp => 'LOG';
+
+    LOG->add(screen => { maxlevel => 'info' });
+    LOG->info("info message");
+
+    # import myapp logger with accessor LOG
+    package MyApp::Foo;
+    use Log::Handler myapp => 'LOG';
+    LOG->info("info message from MyApp::Foo");
+
+    # get myapp logger with get_logger()
+    package MyApp::Bar;
+    use Log::Handler;
+    my $log = Log::Handler->get_logger('myapp');
+    $log->info("info message from MyApp::Bar");
 
 =head1 DESCRIPTION
 
@@ -125,11 +145,26 @@ This option is not used by default.
 
 =item B<newline>
 
-This helpful option appends a newline to the message if a newline not exist.
+C<newline> is a very helpful option. It let the logger appends a newline to
+the message if a newline doesn't exist.
 
     0 - do nothing (default)
     1 - append a newline if not exist
 
+Example:
+
+    $log->add(
+        screen => {
+            newline  => 1,
+            maxlevel => 'info',
+        }
+    );
+
+    $log->info("message\n");
+    $log->info("message");
+
+In both cases the message would be logged with a newline at the end.
+
 =item B<message_layout>
 
 With this option it's possible to create your own message layout with different
@@ -140,6 +175,8 @@ placeholders in C<printf()> style. The available placeholders are:
     %D   Date (option dateformat)
     %P   PID
     %H   Hostname
+    %U   User name
+    %G   Group name
     %N   Newline
     %S   Program name
     %C   Caller - filename and line number
@@ -148,7 +185,7 @@ placeholders in C<printf()> style. The available placeholders are:
     %l   Caller - line number
     %s   Caller - subroutine name
     %r   Runtime in seconds since program start
-    %t   Time measurement - replaced with the time since the last call of $level
+    %t   Time measurement - replaced with the time since the last call of $log->$level
     %m   Message
     %%   Procent
 
@@ -182,8 +219,8 @@ You can create your own placeholders with the method C<set_pattern()>.
 
 This option is just useful if you want to forward messages to output
 modules that needs the parts of a message as a hash reference - as
-example L<Log::Handler::Output::Forward>, L<Log::Handler::Output::DBI>
-or L<Log::Handler::Output::Screen>.
+example L<Log::Handler::Output::Forward>, L<Log::Handler::Output::DBI>,
+L<Log::Handler::Output::Email> or L<Log::Handler::Output::Screen>.
 
 The option expects a list of placeholders:
 
@@ -192,7 +229,7 @@ The option expects a list of placeholders:
 
     # or as a string
     message_pattern => '%T %L %H %m'
- 
+
 The patterns will be replaced with real names as hash keys.
 
     %L   level
@@ -200,6 +237,8 @@ The patterns will be replaced with real names as hash keys.
     %D   date
     %P   pid
     %H   hostname
+    %U   user
+    %G   group
     %N   newline
     %r   runtime
     %C   caller
@@ -236,6 +275,38 @@ Here a full code example:
         print "Message:   $msg->{message}\n";
     }
 
+=item B<prepare_message>
+
+C<prepare_message> is useful if you want to do something with the message before
+it will be logged... maybe you want to create your own layout because message_layout
+doesn't meet your claim.
+
+    $log->add(
+        screen => {
+            newline => 1,
+            message_layout  => '%m (%t)',
+            message_pattern => [ qw/%T %L %H %m/ ],
+            prepare_message => \&format,
+        }
+    );
+
+    $log->error("foo");
+    $log->error("bar");
+    $log->error("baz");
+
+    sub format {
+        my $m = shift;
+
+        $m->{message} = sprintf('%-20s %-20s %-20s %s',
+            $m->{time}, $m->{level}, $m->{hostname}, $m->{message});
+    }
+
+The output looks like
+
+    Mar 08 15:14:20      ERROR                h1434036             foo (0.039694)
+    Mar 08 15:14:20      ERROR                h1434036             bar (0.000510)
+    Mar 08 15:14:20      ERROR                h1434036             baz (0.000274)
+
 =item B<priority>
 
 With this option you can set the priority of your output objects. This means
@@ -245,16 +316,16 @@ increased +1 with each output. Example:
 
 We add a output with no priority
 
-    $log->add(file => { filename => 'file.log' });
+    $log->add(file => { filename => 'file1.log' });
 
 This output gets the priority of 10. Now we add another output
 
-    $log->add(file => { filename => 'file.log' });
+    $log->add(file => { filename => 'file2.log' });
 
 This output gets the priority of 11... and so on.
 
-Messages would be logged at first to the output with the priority of 10 and to
-the output with the priority of 11. Now you can add another output and set the
+Messages would be logged at first to the output with the priority of 10 and then
+to the output with the priority of 11. Now you can add another output and set the
 priority to 1.
 
     $log->add(screen => { dump => 1, priority => 1 });
@@ -335,7 +406,7 @@ NOTE that re-eval in regexes is not valid! Something like
 
 would cause an error!
 
-=item C<filter_caller>
+=item B<filter_caller>
 
 You can use this option to set a package name. Only messages from this
 packages will be logged.
@@ -347,9 +418,9 @@ Example:
     $log->add(screen => {
         maxlevel => 'info',
         newline  => 1,
-        filter_caller  => qr/^Foo::Bar$/,
+        filter_caller  => qr/^Foo::Bar\z/,
         # or
-        # filter_caller => '^Foo::Bar$',
+        # filter_caller => '^Foo::Bar\z',
     });
 
     package Foo::Bar;
@@ -362,6 +433,14 @@ Example:
 
 This would only log the message from the package C<Foo::Bar>.
 
+=item B<except_caller>
+
+This option is just the opposite of C<filter_caller>.
+
+If you want to log messages from all callers but C<Foo::Bar>:
+
+    except_caller => qr/^Foo::Bar\z/
+
 =item B<alias>
 
 You can set an alias if you want to get the output object later. Example:
@@ -414,7 +493,7 @@ The line mode looks like this:
 
 Output:
 
-    Apr 26 12:54:11 [WARNING] 
+    Apr 26 12:54:11 [WARNING]
        CALL(4): package(main) filename(./trace.pl) line(15) subroutine(main::test2) hasargs(0)
        CALL(3): package(main) filename(./trace.pl) line(13) subroutine(main::test1) hasargs(0)
        CALL(2): package(main) filename(./trace.pl) line(12) subroutine(Log::Handler::__ANON__) hasargs(1)
@@ -427,7 +506,7 @@ The same code example but the debugger in block mode would looks like this:
 
 Output:
 
-   Apr 26 12:52:17 [DEBUG] 
+   Apr 26 12:52:17 [DEBUG]
       CALL(4):
          package     main
          filename    ./trace.pl
@@ -601,7 +680,7 @@ Both calls would log - if level INFO is active:
 
 =item B<is_notice()>
 
-=item B<is_warning()>
+=item B<is_warning()>, B<is_warn()>
 
 =item B<is_error()>, B<is_err()>
 
@@ -613,7 +692,7 @@ Both calls would log - if level INFO is active:
 
 =back
 
-These thirteen methods could be very useful if you want to kwow if the current
+These twelve methods could be very useful if you want to kwow if the current
 level would log the message. All methods returns TRUE if the current set
 of C<minlevel> and C<maxlevel> would log the message and FALSE if not.
 
@@ -625,7 +704,7 @@ For a full list take a look into the documentation of L<Log::Handler::Levels>.
 
 =head2 output()
 
-Call C<output($alias)> to get the output object that you added with 
+Call C<output($alias)> to get the output object that you added with
 the option C<alias>.
 
 It's possible to access a output directly:
@@ -729,7 +808,7 @@ Then you can use this pattern in your message layout:
     $log->add(file => {
         filename        => 'file.log',
         mode            => 'append',
-        message_layout  => '%X %m %N',
+        message_layout  => '%X %m%N',
     });
 
 Or use it with C<message_pattern>:
@@ -746,10 +825,115 @@ Or use it with C<message_pattern>:
 
 Note: valid character for the key name are: C<[%\w\-\.]+>
 
+=head2 set_level()
+
+With this method it's possible to change the log level at runtime.
+
+To change the log level it's neccessary to use a alias - see option C<alias>.
+
+    $log->set_level(
+        $alias => { # option alias
+            minlevel => $new_minlevel,
+            maxlevel => $new_maxlevel,
+        }
+    );
+
+=head2 create_logger()
+
+C<create_logger()> is the same like C<new()> but it creates a global
+logger.
+
+    my $log = Log::Handler->create_logger('myapp');
+
+If you want to create more than one object you can call
+
+    my @logger = Log::Handler->create_logger('myapp1', 'myapp2', ...);
+
+=head2 get_logger()
+
+With C<get_logger()> it's possible to get a logger that was created
+with C<create_logger()> or with
+
+    use Log::Handler 'myapp';
+
+Just call
+
+    my $log = Log::Handler->get_logger('myapp');
+
+Or
+
+    my @logger = Log::Handler->get_logger('myapp1', 'myapp2', ...);
+
+=head1 GLOBAL LOG HANDLER
+
+Since version C<0.50> it's possible to define a application logger.
+This means to create a C<Log::Handler> object and import it into
+all modules of your program.
+
+    use Log::Handler alias => accessor;
+
+The accessor is created into the namespace of the caller and
+referenced to the alias.
+
+If the alias doesn't exists then a new C<Log::Handler> object
+will be created.
+
+If no accesseor is set then no accessor is exported into the
+namespace of the caller.
+
+Example:
+
+    package MyAPP;
+    use strict;
+    use warnings;
+    use Log::Handler myapp => 'LOG';
+
+    LOG->config(config => 'myapp.conf');
+    LOG->info('hello world');
+
+Now you can access the logger after you import it.
+
+    package MyAPP::Foo; 
+    use Log::Handler myapp => 'LOG';
+
+    LOG->info('message');
+
+If you want to import another accessor name ...
+
+    package MyAPP::Bar;
+    use Log::Handler myapp => 'MYLOG';
+
+    MYLOG->info('message');
+
+Another way to get the logger is to call C<get_logger()> if you
+don't want to create an accessor.
+
+    package MyAPP::Foo::Bar;
+    use Log::Handler;
+
+    my $log = Log::Handler->get_logger('myapp');
+    $log->info('message');
+
+For a little example you can take a look into the examples directory
+of the distribution (Log-Handler-$VERSION/examples/logger.pl).
+
 =head1 EXAMPLES
 
 L<Log::Handler::Examples>
 
+=head1 BENCHMARK
+
+Run examples/benchmark/benchmark.pl on a Dual-Core AMD Opteron(tm) Processor 1212 HE (1000 MHz)
+
+    simple pattern output took     :  2 wallclock secs ( 2.29 usr +  0.02 sys =  2.31 CPU) @ 43290.04/s (n=100000)
+    default pattern output took    :  4 wallclock secs ( 3.36 usr +  0.76 sys =  4.12 CPU) @ 24271.84/s (n=100000)
+    complex pattern output took    :  6 wallclock secs ( 4.85 usr +  0.91 sys =  5.76 CPU) @ 17361.11/s (n=100000)
+    message pattern output took    :  5 wallclock secs ( 4.31 usr +  0.82 sys =  5.13 CPU) @ 19493.18/s (n=100000)
+    suppressed output took         :  0 wallclock secs ( 0.17 usr +  0.00 sys =  0.17 CPU) @ 588235.29/s (n=100000)
+    filtered caller output took    :  5 wallclock secs ( 3.97 usr +  0.81 sys =  4.78 CPU) @ 20920.50/s (n=100000)
+    suppressed caller output took  :  1 wallclock secs ( 1.29 usr +  0.00 sys =  1.29 CPU) @ 77519.38/s (n=100000)
+    filtered messages output took  :  5 wallclock secs ( 3.91 usr +  0.84 sys =  4.75 CPU) @ 21052.63/s (n=100000)
+
 =head1 EXTENSIONS
 
 Send me a mail if you have questions.
@@ -758,7 +942,6 @@ Send me a mail if you have questions.
 
 Prerequisites for all modules:
 
-    Perl 5.6.1
     Carp
     Data::Dumper
     Devel::Backtrace
@@ -807,7 +990,7 @@ If you send me a mail then add Log::Handler into the subject.
 
 =head1 COPYRIGHT
 
-Copyright (C) 2007 by Jonny Schulz. All rights reserved.
+Copyright (C) 2007-2008 by Jonny Schulz. All rights reserved.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
@@ -849,7 +1032,7 @@ use Log::Handler::Config;
 use Log::Handler::Pattern;
 use base qw(Log::Handler::Levels);
 
-our $VERSION = '0.45';
+our $VERSION = '0.56';
 our $ERRSTR  = '';
 
 # $TRACE and $CALLER_LEVEL are both used as global
@@ -858,11 +1041,13 @@ our $ERRSTR  = '';
 #
 # $TRACE is used to turn on/off tracing.
 #
-# $CALLER_LEVEL is used to determine the current
-# caller level
+# $CALLER_LEVEL is used to determine the current caller level
 our $CALLER_LEVEL = 0;
 our $TRACE        = 0;
 
+# safe logger by app
+my %LOGGER;
+
 # Some constants...
 use constant PRIORITY => 10;
 use constant BOOL_RX  => qr/^[01]\z/;
@@ -881,25 +1066,25 @@ use constant LEVEL_RX => qr/^(?:
 )\z/x;
 
 # to convert minlevel and maxlevel to a number
-our %LEVEL_BY_STRING = ( 
-    DEBUG     =>  7,  
-    INFO      =>  6,  
-    NOTICE    =>  5,  
-    WARNING   =>  4,  
-    WARN      =>  4,  
-    ERROR     =>  3,  
-    ERR       =>  3,  
-    CRITICAL  =>  2,  
-    CRIT      =>  2,  
-    ALERT     =>  1,  
-    EMERGENCY =>  0,  
-    EMERG     =>  0,  
+our %LEVEL_BY_STRING = (
+    DEBUG     =>  7,
+    INFO      =>  6,
+    NOTICE    =>  5,
+    WARNING   =>  4,
+    WARN      =>  4,
+    ERROR     =>  3,
+    ERR       =>  3,
+    CRITICAL  =>  2,
+    CRIT      =>  2,
+    ALERT     =>  1,
+    EMERGENCY =>  0,
+    EMERG     =>  0,
     FATAL     =>  0,
 );
 
 # to iterate from minlevel to maxlevel and
 # create an HoA with all active levels
-my @LEVEL_BY_NUM = qw(
+our @LEVEL_BY_NUM = qw(
     EMERGENCY
     ALERT
     CRITICAL
@@ -921,6 +1106,53 @@ our %AVAILABLE_OUTPUTS = (
     socket  => 'Log::Handler::Output::Socket',
 );
 
+# use Log::Handler foo => 'LOGFOO', bar => 'LOGBAR';
+# use Log::Handler qw/foo LOGFOO bar LOGBAR/;
+sub import {
+    return unless @_ > 1;
+    my $class  = shift;
+    my %create = @_ > 1 ? @_ : (@_, undef);
+    my $caller = (caller)[0];
+
+    while (my ($appl, $export) = each %create) {
+        my $logger = ();
+
+        if (!exists $LOGGER{$appl}) {
+            $LOGGER{$appl} = __PACKAGE__->new();
+        }
+
+        if ($export) {
+            no strict 'refs';
+            my $method = $caller.'::'.$export;
+            *{$method} = sub { $LOGGER{$appl} };
+        }
+    }
+}
+
+sub get_logger {
+    @_ > 1 || croak 'Usage: Log::Handler->get_logger($app)';
+    my $class = shift;
+
+    foreach my $logger (@_) {
+        if (!exists $LOGGER{$logger}) {
+            croak "logger '$logger' does not exists";
+        }
+    }
+
+    return @LOGGER{@_};
+}
+
+sub create_logger {
+    @_ > 1 || croak 'Usage: Log::Handler->create_logger($app)';
+    my $class = shift;
+
+    foreach my $logger (@_) {
+        $LOGGER{$logger} = __PACKAGE__->new();
+    }
+
+    return @LOGGER{@_};
+}
+
 sub new {
     my $class = shift;
 
@@ -1017,7 +1249,7 @@ sub set_pattern {
     my $pattern = shift;
 
     # If no $name is set then we use $pattern as name
-    my ($name, $proto) = @_ == 2 ? @_ : ($pattern, @_);
+    my ($name, $code) = @_ == 2 ? @_ : ($pattern, @_);
 
     if ($pattern !~ /^%[a-ln-z]\z/i) {
         Carp::croak "invalid pattern '$pattern'";
@@ -1030,8 +1262,69 @@ sub set_pattern {
     # Structure:
     #   $self->{pattern}->{'%X'}->{name} = 'name-of-x';
     #   $self->{pattern}->{'%X'}->{code} = 'value-of-x';
-    $self->{pattern}->{$pattern}->{code} = $proto;
     $self->{pattern}->{$pattern}->{name} = $name;
+    $self->{pattern}->{$pattern}->{code} = $code;
+}
+
+sub set_level {
+    @_ == 3 or Carp::croak 'Usage: $log->set_level( $alias => { minlevel => $min, maxlevel => $max } )';
+    my ($self, $name, $new) = @_;
+    my $alias = $self->{alias};
+
+    if (!exists $alias->{$name}) {
+        Carp::croak "alias '$name' does not exists";
+    }
+
+    if (ref($new) ne 'HASH') {
+        Carp::croak "the second parameter to set_level() must be a hash reference";
+    }
+
+    if (!defined $new->{minlevel} && !defined $new->{maxlevel}) {
+        Carp::croak "no new level given to set_level()";
+    }
+
+    foreach my $level (qw/minlevel maxlevel/) {
+        next unless defined $new->{$level};
+
+        if ($new->{$level} =~ LEVEL_RX) {
+            $alias->{$name}->{$level} = $new->{$level};
+            next if $new->{$level} =~ /^\d\z/;
+            $new->{$level} = uc($new->{$level});
+            $new->{$level} = $LEVEL_BY_STRING{ $new->{$level} };
+            $alias->{$name}->{$level} = $new->{$level};
+        } else {
+            Carp::croak "invalid level set to set_level()";
+        }
+    }
+
+    $alias->{$name}->{levels} = { };
+    my $levels = $self->{levels} = { };
+
+    foreach my $level_num ($alias->{$name}->{minlevel} .. $alias->{$name}->{maxlevel}) {
+        my $level = $LEVEL_BY_NUM[ $level_num ];
+        $alias->{$name}->{levels}->{$level} = 1;
+
+        if ($level_num < 4) {
+            $alias->{$name}->{levels}->{FATAL} = 1;
+        }
+    }
+
+    foreach my $output (@{ $self->{outputs} }) {
+        foreach my $level (keys %{$output->{levels}}) {
+            if ($levels->{$level}) {
+                my @old_order = @{$levels->{$level}};
+                $levels->{$level} = [
+                    map  { $_->[0] }
+                    sort { $a->[1] <=> $b->[1] }
+                    map  { [ $_, $_->{priority} ] } @old_order
+                ];
+            } else {
+                push @{$levels->{$level}}, $output;
+            }
+        }
+    }
+
+    return 1;
 }
 
 sub output {
@@ -1078,9 +1371,9 @@ sub errstr { $ERRSTR }
 # private stuff
 #
 
-sub _shift_options {
-    my ($self, $output_opts) = @_;
-    my %handler_opts;
+sub _split_options {
+    my ($self, $opts) = @_;
+    my (%handler_opts, %output_opts);
 
     # It's possible to pass all options for the handler and for the
     # output to add(). These options must be splitted. The options
@@ -1088,7 +1381,7 @@ sub _shift_options {
     # options for the output will be passed - as example - to
     # Log::Handler::Output::File.
 
-    my @shift_options = qw(
+    my %split_options = map { $_ => 0 } qw(
         alias
         debug_mode
         debug_skip
@@ -1097,22 +1390,26 @@ sub _shift_options {
         filter
         filter_message
         filter_caller
+        except_caller
         maxlevel
         message_layout
         message_pattern
+        prepare_message
         minlevel
         newline
-        prepare_message
         priority
         timeformat
     );
 
-    foreach my $opt ( @shift_options ) {
-        next unless exists $output_opts->{$opt};
-        $handler_opts{$opt} = delete $output_opts->{$opt};
+    while (my ($k, $v) = each %$opts) {
+        if (exists $split_options{$k}) {
+            $handler_opts{$k} = $v;
+        } else {
+            $output_opts{$k} = $v;
+        }
     }
 
-    return \%handler_opts;
+    return (\%handler_opts, \%output_opts);
 }
 
 sub _new_output {
@@ -1120,7 +1417,7 @@ sub _new_output {
     my $type    = shift;
     my $args    = @_ > 1 ? {@_} : shift;
     my $package = ref($type);
-    my ($output, $handler_opts);
+    my ($output, $handler_opts, $output_opts);
 
     # There are two ways to add an output:
     #
@@ -1137,9 +1434,8 @@ sub _new_output {
         $output = $type;
         $handler_opts = $args;
     } else {
-        # Shift the handler options from $args. The rest in %$args
-        # will be passed to the output.
-        $handler_opts = $self->_shift_options($args);
+        # Split the handler and output options from $args.
+        ($handler_opts, $output_opts) = $self->_split_options($args);
 
         # Try to determine which output is wanted...
         if (exists $AVAILABLE_OUTPUTS{$type}) {
@@ -1151,7 +1447,8 @@ sub _new_output {
         }
 
         $package->require;
-        $output = $package->new($args) or Carp::croak $package->errstr;
+        $output = $package->new($output_opts)
+            or Carp::croak $package->errstr;
     }
 
     $handler_opts = $self->_validate_options($handler_opts);
@@ -1160,9 +1457,10 @@ sub _new_output {
 
 sub _validate_options {
     my ($self, @args) = @_;
+    my %wanted  = ();
     my $pattern = $self->{pattern};
-    my (%wanted, $is_fatal);
 
+    # Option 'filter' is deprecated.
     if (exists $args[0]{filter}) {
         $args[0]{filter_message} = delete $args[0]{filter};
     }
@@ -1244,6 +1542,10 @@ sub _validate_options {
             type => Params::Validate::SCALAR | Params::Validate::SCALARREF,
             optional => 1,
         },
+        except_caller => {
+            type => Params::Validate::SCALAR | Params::Validate::SCALARREF,
+            optional => 1,
+        },
     });
 
     if ($options{filter_message}) {
@@ -1267,7 +1569,7 @@ sub _validate_options {
     foreach my $level_num ($options{minlevel} .. $options{maxlevel}) {
         my $level = $LEVEL_BY_NUM[ $level_num ];
         $options{levels}{$level} = 1;
-        next if $is_fatal || $level_num > 3;
+        next if $level_num > 3;
         $options{levels}{FATAL} = 1;
     }
 
@@ -1306,7 +1608,7 @@ sub _validate_options {
         #
         #   sub {
         #       my ($w, $m) = @_; # %wanted pattern, %message
-        #       $m->{'message'} = 
+        #       $m->{'message'} =
         #           $w->{'time'}
         #           . ' ['
         #           . $w->{'level'}
@@ -1405,6 +1707,8 @@ sub _validate_filter {
             Carp::croak "missing condition for paramater 'filter'";
         }
 
+        # Remove all valid characters from the condition
+        # and check if invalid characters left.
         my $cond = $filter{condition};
         $cond =~ s/match\d+//g;
         $cond =~ s/[()&|!<>=\s\d]+//;
